#!/usr/bin/env python
#-*- encoding: utf-8 -*-

"""
Desktop application for downloading and playing songs provided
by the grooveshark.com service
"""

import os
import sys
import random
import dbus
import dbus.service
import dbus.glib
from lib.tfuncs import InitThread
from lib.tfuncs import DownloadThread
from lib.tfuncs import PlayThread
from lib.tfuncs import SearchThread
from lib.tfuncs import KeyListenerThread
from lib.tfuncs import LyricsThread
from lib.tfuncs import UpdateThread
from lib.tfuncs import SongCoverThread
import lib.guihelpers
import locale
import gettext
from gettext import gettext as _
import urllib2
import webbrowser
import pickle

try:
    import pygtk
    pygtk.require('2.0')
    import glib
    import gtk
    import gobject
except ImportError:
    print "You don't have python-gtk2 installed!"
    sys.exit(1)

try:
    import pynotify
    HAVE_NOTIFY = True
except ImportError:
    print "You don't have pynotify installed!"
    HAVE_NOTIFY = False

try:
    import pygst
    pygst.require('0.10')
    import gst
except ImportError:
    print "You don't have python-gstreamer installed!"
    sys.exit(1)

try:
    import pylast
    HAVE_PYLAST = True
except ImportError:
    print "You need to install pylast: sudo pip install pylast"
    HAVE_PYLAST = False


try:
    from configobj import ConfigObj
except ImportError:
    print "You don't have python-configobj installed!"
    sys.exit(1)

BASEPATH = os.path.abspath(os.path.dirname(sys.argv[0]))

APP = 'gsharkdown'
if os.path.exists("locale"):
    DIR = "%s/locale" % BASEPATH
else:
    DIR = os.path.join(sys.prefix, 'share', 'locale')

try:
    locale.setlocale(locale.LC_ALL, '')
except locale.Error:
    locale.setlocale(locale.LC_ALL, 'en_US.utf8')
locale.bindtextdomain(APP, DIR)
locale.bind_textdomain_codeset(APP, 'UTF-8')
gettext.bindtextdomain(APP, DIR)
gettext.textdomain(APP)
gettext.install(APP, localedir = DIR, unicode = True)

LAST_KEY = "51fd71dc8939360b25a1029e556258a4"
LAST_SECRET = "cf35dd38f998ca4d3af4adbe59ae23f7"

if os.path.exists("%s/.gsharkdown/gsharkdown.ini" % os.environ.get("HOME")):
    CONFIG = ConfigObj("%s/.gsharkdown/gsharkdown.ini" % os.environ.get("HOME"))
    try:
        if HAVE_PYLAST:
            test = CONFIG['scrobbling']
        else:
            raise KeyError()
    except KeyError:
        CONFIG['scrobbling'] = 0
        CONFIG['lastuser'] = ""
        CONFIG['lastpass'] = ""

    try:
        if HAVE_NOTIFY:
            test = CONFIG['show_notification']
        else:
            raise KeyError()
    except KeyError:
        CONFIG['show_notification'] = 0

    try:
        test = CONFIG['quit_without_confirmation']
    except KeyError:
        CONFIG['quit_without_confirmation'] = 1

    try:
        test = CONFIG['startup_update_check']
    except KeyError:
        CONFIG['startup_update_check'] = 1
    try:
        test = CONFIG['speed_limit']
    except:
        CONFIG['speed_limit'] = 0

    try:
        test = CONFIG['file_pattern']
    except KeyError:
        CONFIG['file_pattern'] = "{artist} - {song}"

    CONFIG.write()

else:
    try:
        os.mkdir("%s/.gsharkdown/" % os.environ.get("HOME"))
    except:
        pass
    CONFIG = ConfigObj("%s/.gsharkdown/gsharkdown.ini" % os.environ.get("HOME"))
    MUSIC_DIR = glib.get_user_special_dir(glib.USER_DIRECTORY_MUSIC) or os.path.join(os.environ.get("HOME"), "Music")
    CONFIG['down_path'] = os.path.join(MUSIC_DIR, "Grooveshark")
    if os.path.isdir(CONFIG['down_path']) == False:
        try:
            os.makedirs(CONFIG['down_path'])
        except:
            pass
    CONFIG['show_stat_icon'] = 0
    CONFIG['repeat_playlist'] = 1
    CONFIG['shuffle_playlist'] = 0
    CONFIG['show_notification'] = 0
    CONFIG['update_checked'] = 0
    CONFIG['completition'] = ""
    CONFIG['scrobbling'] = 0
    CONFIG['lastuser'] = ""
    CONFIG['lastpass'] = ""
    CONFIG['startup_update_check'] = 1
    CONFIG['quit_without_confirmation'] = 1
    CONFIG['speed_limit'] = 0
    CONFIG['file_pattern'] = "{artist} - {song}"
    CONFIG.write()


class SharkDown:
    """
    The main application class for gSharkDown
    """
    working = None
    playing = None
    # just to prevend scrobbeling more then once for 
    # track.
    scrobbled = 0
    # Download dictionary in form Filename : Thread
    downqueue = []

    last_iter = None

    def __init__(self):
        """
        Initialise the main application window
        """
        gobject.threads_init()
        builder = gtk.Builder()
        builder.set_translation_domain(APP)
        builder.add_from_file('%s/data/main_window.ui' % BASEPATH)
        self.window = builder.get_object('mainwindow')
        self.window.connect('delete-event', self.window_close)
        self.windowstate = 1
        self.current_song = None
        self.play_button = builder.get_object('toolbutton_play')
        self.ff_button = builder.get_object('toolbutton_play_next')
        self.prev_button = builder.get_object('toolbutton_play_previous')
        self.lyrics_button = builder.get_object('toolbutton_lyrics')
        self.songinfo = builder.get_object('songinfo')
        self.songinfo.set_text("")
        self.prefsmenu = builder.get_object('mainmenu')
        self.lovebutton = builder.get_object('toolbutton3')
        repeat_button = builder.get_object('togglebutton_repeat')
        repeat_button.set_active(int(CONFIG['repeat_playlist']))
        shuffle_button = builder.get_object('togglebutton_shuffle')
        shuffle_button.set_active(int(CONFIG['shuffle_playlist']))
        self.player = gst.element_factory_make("playbin", "player")

        # Completition configuration
        self.entry = builder.get_object('toolentry_search')
        completition = gtk.EntryCompletion()
        comp_store = gtk.ListStore(str)
        completition.set_model(comp_store)
        for item in CONFIG['completition'].split("|"):
            comp_store.append([item])
        completition.set_minimum_key_length(1)
        completition.set_text_column(0)
        self.entry.set_completion(completition)

        # Results List definition
        # Model: Title, Artist, Album, SongID, Song Array
        self.result = gtk.ListStore(str, str, str, str, object)
        self.result_view = builder.get_object('list_results')
        self.result_view.set_model(self.result)
        self.result_view.connect("row-activated", self.on_result_row_activated)
        self.result_view.get_selection().set_mode(gtk.SELECTION_MULTIPLE)
        self._create_search_columns()

        # Play List definition
        # Model: Title, Artist, SongID, Song Array, Font Weight (bold on playing)
        self.sw2 = builder.get_object('scrolledwindow2')
        self.playlist = gtk.ListStore(str, gtk.gdk.Pixbuf, str, str, object, int)
        self.play_view = gtk.IconView(self.playlist)
        self.play_view.set_markup_column(0)
        self.play_view.set_pixbuf_column(1)
        self.play_view.set_tooltip_column(2)
        self.play_view.set_item_width(120);
        self.play_view.set_reorderable(True)
        self.play_view.set_columns(999)
        self.play_view.connect('item-activated', self.double_click_start)
        self.play_view.show()
        self.sw2.add(self.play_view)
        #self._create_play_columns()
        #self.selector = self.play_view.get_selection()

        # Download list definition
        # Model: File Name, SongID, Progress, Size, Thread Object
        self.downloads = gtk.ListStore(str, str, int, str, object)
        self.downloads_view = builder.get_object('list_downloads')
        self.downloads_view.set_model(self.downloads)
        self.downloads_view.hide_all()
        self._create_downloads_columns()
        self.downloads_count = builder.get_object('downloads_count')
        self.downloads_expander = builder.get_object('expander_download')
        self.update_downloads_count()
        self.downmenu = builder.get_object('downloadmenu')

        self.staticon = lib.guihelpers.GsharkIndicator(self)

        if os.path.exists("%s/.gsharkdown/playlist.pkl" % os.environ.get("HOME")):
            self.load_saved_playlist("%s/.gsharkdown/playlist.pkl" % os.environ.get("HOME"))
        else:
            print "Playlist not found"

        if os.path.exists("%s/.gsharkdown/downqueue.pkl" % os.environ.get("HOME")):
            self.load_downqueue_list("%s/.gsharkdown/downqueue.pkl" % os.environ.get("HOME"))

        if int(CONFIG['show_notification']) == 1 and HAVE_NOTIFY:
            pynotify.init("gSharkDown")

        # Set default directory if is empty
        if CONFIG['down_path'] == "":
            CONFIG['down_path'] = os.path.join(glib.get_user_special_dir(glib.USER_DIRECTORY_MUSIC), "Grooveshark")

        # Scrobbling initialisation
        self.lastfm = ""
        if int(CONFIG['scrobbling']) == 1 and HAVE_PYLAST:
            try:
                self.lastfm = pylast.LastFMNetwork(api_key = LAST_KEY,
                                          api_secret = LAST_SECRET,
                                          username = CONFIG['lastuser'],
                                          password_hash = CONFIG['lastpass'])
                self.lovebutton.set_sensitive(True)
            except pylast.WSError:
                error = lib.guihelpers.ErrorMessage(self.window,
                            _("Check your password and username for Last.fm"))
                error.show_all()
                pass

        builder.connect_signals(self)
        self.tlisten = KeyListenerThread(self)
        self.tlisten.start()
        if CONFIG['update_checked'] == 0:
            self.check_for_update(None)
        else:
            if CONFIG['startup_update_check'] == 1:
                self.check_for_update(None)
        self.window.show_all()

    def get_playing_iter(self):
        iter = self.playlist.get_iter_first()
        while iter != None:
            if self.playlist[iter][5] > 400:
                return iter
            iter = self.playlist.iter_next(iter)

        return None

    def get_iter_last(self, model):
        """
        Get the last iter from a model
        """
        rows = model.iter_n_children(None);
        return model.get_iter([rows - 1]);

    def result_song(self, index):
        return self.result[index][4]

    def playlist_song(self, index):
        return self.playlist[index][4]

    def show_prefs_menu(self, button, data = None):
        self.prefsmenu.popup(None, None, None, 0, 0)

    def add_song_to_playlist(self, song):
        song_string = "<span>%s</span>\n<span fgcolor='#777777'>%s</span>" % (self.get_sliced_string(glib.markup_escape_text(song['SongName']), 25),
                                                                              self.get_sliced_string(glib.markup_escape_text(song['ArtistName']), 20))
        #if len(song_string) > 14:
        #    song_string = song_string[:13] + "..."
        tooltip = _("<b>Title:</b> {title}\n").format(title = glib.markup_escape_text(song['SongName']))
        tooltip += _("<b>Artist:</b> {artist}\n").format(artist = glib.markup_escape_text(song['ArtistName']))
        tooltip += _("<b>Album:</b> {album}\n").format(album = glib.markup_escape_text(song['AlbumName']))
        tooltip += _("<b>Year:</b> {year}").format(year = song['Year'])

        appended_iter = self.playlist.append([
            song_string,
            self.create_loading_track_icon(),
            tooltip,
            song['SongID'],
            song,
            400
        ])

        thread = SongCoverThread(self, appended_iter)
        thread.start()

    def create_default_track_icon(self):
        default = gtk.gdk.pixbuf_new_from_file("%s/data/sdefault.png" % BASEPATH)
        return self.create_cornered_image(default)

    def create_loading_track_icon(self):
        default = gtk.gdk.pixbuf_new_from_file("%s/data/loading.png" % BASEPATH)
        return self.create_cornered_image(default)

    def create_cornered_image(self, pixbuf):
        corners = gtk.gdk.pixbuf_new_from_file("%s/data/corners.png" % BASEPATH)
        corners.composite(pixbuf, 0, 0, pixbuf.props.width, pixbuf.props.height,
                          0, 0, 1.0, 1.0, gtk.gdk.INTERP_HYPER, 255)
        return pixbuf

    def recover_icon(self, cover_url):
        try:
            url = "http://beta.grooveshark.com/static/amazonart/s%s"
            response = urllib2.urlopen(url % cover_url)
            loader = gtk.gdk.PixbufLoader()
            loader.write(response.read())
            loader.close()
            image = loader.get_pixbuf()
            return image
        except:
            image = self.create_default_track_icon()
            return image

    def get_current_cover_url(self, iter):
        song = self.playlist.get_value(self.last_iter, 4)
        try:
            return song['CoverArtFilename']
        except:
            return "sdefault"


    def load_saved_playlist(self, path):
        """
        Loads the saved playlist from .pkl file
        """
        file = open(path, 'rb')
        eoferror = True
        playitems = []

        while eoferror:
            try:
                playitems.append(pickle.load(file))
            except:
                eoferror = False

        for i in range(0, len(playitems)):
            self.add_song_to_playlist(playitems[i])

    def load_downqueue_list(self, path):
        """
        Loads the saved playlist from the .pkl file
        """
        file = open(path, 'rb')
        eoferror = True
        downitems = []

        while eoferror:
            try:
                downitems.append(pickle.load(file))
            except:
                eoferror = False

        for i in range(0, len(downitems)):
            tDownload = DownloadThread(self,
                                       downitems[i]['song'],
                                       downitems[i]['filename'])
            tDownload.start()

    def save_playlist(self, path):
        """
        Saves the playlist from the playlist treeview to 
        /home/$USER/.gsharkdown/playlist.pkl
        """
        try:
            output = open(path, 'w')
            for i in range(0, len(self.playlist)):
                pickle.dump(self.playlist_song(i), output)
            output.close()
        except:
            error = lib.guihelpers.ErrorMessage(None,
                            _("Error saving the playlist"))
            error.show_all()

    def save_downwueue(self, path):
        """
        Saves the download queue to pickle file in
        /home/$USER/.gsharkdown/downqueue.pkl
        """
        try:
            output = open(path, 'w')
            for i in range(0, len(self.downqueue)):
                pickle.dump(self.downqueue[i], output, -1)
            output.close()
        except:
            error = lib.guihelpers.ErrorMessage(None,
                            _("Error saving the download queue"))
            error.show_all()


    def save_as_playlist(self, widget, data = None):
        """
        Saves the playlist to .pkl file in directory
        choosen from the user
        """
        savedlg = gtk.FileChooserDialog(_("Save playlist to file"),
                                       None,
                                       gtk.FILE_CHOOSER_ACTION_SAVE,
                                       (gtk.STOCK_CANCEL, gtk.RESPONSE_REJECT,
                                       gtk.STOCK_SAVE, gtk.RESPONSE_OK))
        response = savedlg.run()

        if response == gtk.RESPONSE_OK:
            filename = savedlg.get_filename()
            if filename[-4:] != ".pkl":
                filename += ".pkl"
            self.save_playlist(filename)
        savedlg.destroy()

    def open_saved_playlist(self, widget, data = None):
        """
        Opens a usersaved playlist from .pkl file
        """
        opendlg = gtk.FileChooserDialog(_("Select Playlist to open"),
                                       None,
                                       gtk.FILE_CHOOSER_ACTION_OPEN,
                                       (gtk.STOCK_CANCEL, gtk.RESPONSE_REJECT,
                                       gtk.STOCK_OPEN, gtk.RESPONSE_OK))
        openfilter = gtk.FileFilter()
        openfilter.set_name(_('Playlist files'))
        openfilter.add_pattern("*.pkl")

        opendlg.add_filter(openfilter)

        response = opendlg.run()

        if response == gtk.RESPONSE_OK:
            self.playlist.clear()
            self.stop_play()
            self.load_saved_playlist(opendlg.get_filename())
            self.play_by_index(0)
        opendlg.destroy()

    def window_close(self, widget, data = None):
        """
        Closes the main window
        """
        self.window.hide_on_delete()
        self.windowstate = 0
        return True

    def check_for_update(self, widget, data = None):
        """
        Checks if a new version of the application is available
        on the application website. NEVER change the VERSION file
        manually!
        """
        othertext = _("To update to the latest version ")
        othertext += _("you can visit the <a href=\"http://https://bitbucket.org/vkolev/gsharkdown/downloads\"> ")
        othertext += _("download site</a>. If you are using the latest version and want to be ")
        othertext += _("informed about new versions, just enable the option in the <b>Preferences</b> dialog.")
        markup = _("<b>New version <span fgcolor=\"red\"><i>%s</i></span> is available</b>")
        nmarkup = _("<b>You are using the latest version!</b>")
        CONFIG['update_checked'] = 1
        CONFIG.write()
        t = UpdateThread(BASEPATH, markup, nmarkup, othertext)
        t.start()

    def icon_clicked(self, widget, data = None):
        """
        Executed on left clicking the statusicon
        """
        if(self.windowstate == 0):
            self.window.show_all()
            self.windowstate = 1
        else:
            self.window.hide_on_delete()
            self.windowstate = 0
            return True

    def _create_search_columns(self):
        """
        Generate the TreeView columns in the main window
        for the search results
        """
        rendererText = gtk.CellRendererText()
        column = gtk.TreeViewColumn(_("Title"), rendererText, text = 0)
        column.set_resizable(True)
        column.set_max_width(300)
        self.result_view.append_column(column)

        rendererText = gtk.CellRendererText()
        column = gtk.TreeViewColumn(_("Artist"), rendererText, text = 1)
        column.set_resizable(True)
        self.result_view.append_column(column)

        rendererText = gtk.CellRendererText()
        column = gtk.TreeViewColumn(_("Album"), rendererText, text = 2)
        column.set_resizable(True)
        column.set_max_width(150)
        self.result_view.append_column(column)

        rendererText = gtk.CellRendererText()
        column = gtk.TreeViewColumn(_("SongID"), rendererText, text = 3)
        column.set_resizable(True)
        self.result_view.append_column(column)

    def _create_downloads_columns(self):
        """
        Generate the TreeView columns in the main window
        for the download list items
        """
        rendererText = gtk.CellRendererText()
        column = gtk.TreeViewColumn(_("File name"), rendererText, text = 0)
        column.set_resizable(True)
        self.downloads_view.append_column(column)

        rendererText = gtk.CellRendererText()
        column = gtk.TreeViewColumn(_("SongID"), rendererText, text = 1)
        column.set_resizable(True)
        self.downloads_view.append_column(column)

        rendererProgress = gtk.CellRendererProgress()
        column = gtk.TreeViewColumn(_("Download progress"), rendererProgress, value = 2)
        column.set_resizable(True)
        self.downloads_view.append_column(column)

        rendererText = gtk.CellRendererText()
        column = gtk.TreeViewColumn(_("Size"), rendererText, text = 3)
        column.set_resizable(True)
        self.downloads_view.append_column(column)

    def quit_app(self, widget, data = None):
        """
        Confirmation dialog when exiting the application
        """
        if CONFIG['quit_without_confirmation'] == '0':
            dialog = gtk.MessageDialog(None, gtk.DIALOG_MODAL,
                                  type = gtk.MESSAGE_QUESTION,
                                  buttons = gtk.BUTTONS_YES_NO,
                                  message_format = _("Are you sure to quit?"))
            dialog.set_title(_("Quit?"))
            response = dialog.run()
            dialog.destroy()
        else:
            response = gtk.RESPONSE_YES

        if response == gtk.RESPONSE_YES:
            self.save_playlist("%s/.gsharkdown/playlist.pkl" % os.environ.get("HOME"))
            self.save_downwueue("%s/.gsharkdown/downqueue.pkl" % os.environ.get("HOME"))
            self.stop_all_downloads()
            gtk.main_quit()

    def copy_song(self, widget, data = None):
        """
        Copy the selected song name from the results list in the
        GNOME Clipboard
        """
        select = self.result_view.get_selection().get_selected_rows()
        if self.result_view.get_selection().count_selected_rows() > 0:
            song = self.result_song(select[1][0])
            copystring = "%s - %s" % (song['ArtistName'], song['SongName'])
            clipboard = gtk.clipboard_get()
            clipboard.set_text(copystring)
            clipboard.store()

    def add_to_playlist(self, widget, data = None):
        """
        Adds selected item from the results to the playlist
        """
        select = self.result_view.get_selection().get_selected_rows()
        for item in select[1]:
            song = self.result_song(item[0])

            self.add_song_to_playlist(song)

    def remove_from_playlist(self, widget, data = None):
        """
        Removes an item from the playlist and the playitems list
        """
        select = self.play_view.get_selected_items()
        selectiter = self.playlist.get_iter(select[0])
        if selectiter != None:
            next = self.playlist.iter_next(selectiter)
            self.playlist.remove(selectiter)
            if selectiter == self.get_playing_iter():
                self.stop_play()
            if next != None:
                self.play_view.select_path((select[0][0] + 1,))

    def clear_playlist(self, widget, data = None):
        """
        Removes all items from the playlist
        """
        if self.last_iter != None:
            self.stop_play()
        self.playlist.clear()

    def toggle_repeat(self, widget, data = None):
        """
        Toggles the configuration option for looping trough the
        playlist
        """
        CONFIG['repeat_playlist'] = int(widget.get_active())
        CONFIG.write()

    def toggle_shuffle(self, widget, data = None):
        """
        Toggle the configuration option for shuffleing the playlist
        """
        CONFIG['shuffle_playlist'] = int(widget.get_active())
        CONFIG.write()

    def double_click_start(self, widget, path, data = None):
        """
        Starts playling on double click event on the playlist
        treeview.
        """
        self.play_view.unselect_all()
        self.play_by_index(path[0])

    def on_result_row_activated(self, path, column, data = None):
        """
        On double click in the search treeview the item is added to 
        the playlist and starts playling.
        """
        song = self.result_song(column[0])
        self.add_song_to_playlist(song)

    def play_next(self, widget, data = None):
        """
        Plays the next item from the playlist
        """
        if self.get_playing_iter() == None:
            self.play_by_index(0)
        else:
            index = self.playlist.get_path(self.get_playing_iter())[0] + 1
            if int(CONFIG['shuffle_playlist']) == 1:
                index = random.randint(0, len(self.playlist) - 1)

            if index >= len(self.playlist):
                if int(CONFIG['repeat_playlist']) == 1:
                    self.play_by_index(0)
                else:
                    self.stop_play()
            else:
                self.play_by_index(index)

    def play_previous(self, widget, data = None):
        """
        Plays the previous song from the playlist
        """
        if self.get_playing_iter() == None:
            self.play_by_index(0)
        else:
            index = self.playlist.get_path(self.get_playing_iter())[0] - 1
            if int(CONFIG['shuffle_playlist']) == 1:
                index = random.randint(0, len(self.playlist) - 1)

            if index < 0:
                if int(CONFIG['repeat_playlist']) == 1:
                    self.play_by_index(len(self.playlist) - 1)
                else:
                    self.play_by_index(0)
            else:
                self.play_by_index(index)

    def play_by_index(self, index):
        """
        Start the play by an index on playlist.
        """
        if index < 0:
            index = 0

        if index >= len(self.playlist):
            index = len(self.playlist) - 1

        iter = self.playlist.get_iter((index,))

        if iter != self.get_playing_iter():
            self.scrobbled = 0
            for item in self.playlist:
                item[5] = 400
            self.playlist[iter][5] = 600
            self.play_button.set_stock_id(gtk.STOCK_MEDIA_STOP)
            self.player.set_state(gst.STATE_NULL)
            song = self.playlist_song(index)
            self.set_songinfos(song)
            t = PlayThread(self, song,
                       int(CONFIG['repeat_playlist']),
                      self.lastfm)
            t.start()

    def stop_play(self):
        """
        Stop current playing
        """
        for item in self.playlist:
            item[5] = 400
        self.player.set_state(gst.STATE_NULL)
        self.play_button.set_stock_id(gtk.STOCK_MEDIA_PLAY)
        self.set_songinfos()

    def on_volume_change(self, widget, data = 0.5):
        """
        Volume change handler
        """
        self.player.set_property("volume", float(data))
        return True

    def show_info(self, widget, data = None):
        """
        Song infromation dialog for first item selected from the search results.
        """
        select = self.result_view.get_selection().get_selected_rows()
        if self.result_view.get_selection().count_selected_rows() > 0:
            song = self.result_song(select[1][0])
            dialog = gtk.MessageDialog(None, gtk.DIALOG_DESTROY_WITH_PARENT,
                                      0, gtk.BUTTONS_OK,)
            dialog.set_title(_("Song Information - {songid}").format(songid = song['SongID']))
            _filename = _("<b>Filename:</b> {artist} - {title}.mp3\n").format(artist = glib.markup_escape_text(song['ArtistName']),
                                                       title = glib.markup_escape_text(song['SongName']))
            _artist = _("<b>Artist:</b> {artist}\n").format(artist = glib.markup_escape_text(song['ArtistName']))
            _title = _("<b>Title:</b> {title}\n").format(title = glib.markup_escape_text(song['SongName']))
            _duration = "%.2f" % (float(song['EstimateDuration']) / 60)
            _duration = _("<b>Duration:</b> {mins}:{secs} min\n").format(mins = _duration.split(".")[0],
                                                      secs = _duration.split(".")[1])
            _album = _("<b>Album:</b> {album}\n").format(album = glib.markup_escape_text(song['AlbumName']))
            _year = _("<b>Year:</b> {year}\n").format(year = song['Year'])
            info = _filename + _artist + _title + _duration + _album + _year
            image = gtk.Image()
            try:
                url = "http://beta.grooveshark.com/static/amazonart/s%s"
                response = urllib2.urlopen(url % song['CoverArtFilename'])
                loader = gtk.gdk.PixbufLoader()
                loader.write(response.read())
                loader.close()
                image.set_from_pixbuf(loader.get_pixbuf())
                dialog.set_image(image)
            except:
                pass
            dialog.set_markup(info)
            dialog.show_all()
            dialog.run()
            dialog.destroy()

    def get_lyrics(self, widget, data = None):
        """
        Tries to retrieve lyrics for selected song from the playlist.
        """
        if self.current_song != None:
            t = LyricsThread(self, '%s/data/lyrics_dialog.ui' % BASEPATH,
                            self.current_song['ArtistName'],
                            self.current_song['SongName'])
            t.start()
            #lyric = lib.lyrdblib.search(song['ArtistName'],
            #                             song['SongName'])
        else:
            info = lib.guihelpers.InfoDialog(self.window,
                _("There should be a playing song to view\nthe lyrics for it"))
            info.show_all()

    def show_about(self, widget, data = None):
        """
        About dialog for the application
        """
        builder = gtk.Builder()
        builder.set_translation_domain(APP)
        builder.add_from_file('%s/data/about_dialog.ui' % BASEPATH)
        flattr = builder.get_object('flattrbut')
        flattr.connect("clicked", self.flattr_this)
        updbtn = builder.get_object('updatbut')
        updbtn.connect("clicked", self.check_for_update)
        about = builder.get_object('aboutdialog1')
        about.run()
        about.destroy()

    def flattr_this(self, widget, data = None):
        """
        Opens the flattr page for gSharkDown
        """
        webbrowser.open("http://flattr.com/thing/275401/gSharkDown")

    def edit_preferences(self, widget, data = None):
        """
        Preferences dialog with save and close buttons
        """
        builder = gtk.Builder()
        builder.set_translation_domain(APP)
        builder.add_from_file('%s/data/preferenes_dialog.ui' % BASEPATH)
        self.prefs = builder.get_object('window1')
        self.dest = builder.get_object('filechooserbutton1')
        self.dest.set_filename(CONFIG['down_path'])
        self.quit_without_confirmation = builder.get_object("checkbutton1")
        self.quit_without_confirmation.set_active(int(CONFIG['quit_without_confirmation']))
        self.bubble = builder.get_object("checkbutton2")
        self.speed = builder.get_object("spinbutton1")
        self.speed.set_value(int(CONFIG['speed_limit']))
        self.file_pattern = builder.get_object("entry1")
        self.file_pattern.set_text(CONFIG['file_pattern'])
        self.startup_check = builder.get_object("checkbutton4")
        self.startup_check.set_active(int(CONFIG['startup_update_check']))
        if HAVE_NOTIFY:
            self.bubble.set_active(int(CONFIG['show_notification']))
        else:
            self.bubble.set_sensitive(False)
        self.lastuser = builder.get_object("entry2")
        self.lastuser.set_text(CONFIG['lastuser'])
        self.lastpass = builder.get_object("entry3")
        self.lastpass.set_visibility(False)
        self.lastpass.set_text(CONFIG['lastpass'])
        self.scrobble = builder.get_object("checkbutton3")
        if HAVE_PYLAST:
            self.scrobble.set_active(int(CONFIG['scrobbling']))
        else:
            self.scrobble.set_sensitive(False)
        self.lastuser.set_sensitive(self.scrobble.get_active())
        self.lastpass.set_sensitive(self.scrobble.get_active())
        builder.connect_signals(self)
        self.prefs.show_all()

    def on_scrobble_toggle(self, widget, data = None):
        self.lastuser.set_sensitive(widget.get_active())
        self.lastpass.set_sensitive(widget.get_active())

    def save_preferences(self, widget, data = None):
        """
        Saves the information from the Preferences dialog in the
        configuration file
        """
        CONFIG['down_path'] = self.dest.get_filename()
        if self.quit_without_confirmation.get_active():
            CONFIG['quit_without_confirmation'] = '1'
        else:
            CONFIG['quit_without_confirmation'] = '0'

        if self.bubble.get_active():
            CONFIG['show_notification'] = 1
        else:
            CONFIG['show_notification'] = 0

        if self.startup_check.get_active():
            CONFIG['startup_update_check'] = 1
        else:
            CONFIG['startup_update_check'] = 0

        CONFIG['file_pattern'] = self.file_pattern.get_text()
        CONFIG['speed_limit'] = self.speed.get_value_as_int()

        if self.scrobble.get_active():
            CONFIG['scrobbling'] = 1
            CONFIG['lastuser'] = self.lastuser.get_text()
            CONFIG['lastpass'] = pylast.md5(self.lastpass.get_text())
        else:
            CONFIG['scrobbling'] = 0
            self.lovebutton.set_sensitive(False)
        CONFIG.write()
        try:
            self.lastfm = pylast.LastFMNetwork(api_key = LAST_KEY,
                                          api_secret = LAST_SECRET,
                                          username = CONFIG['lastuser'],
                                          password_hash = CONFIG['lastpass'])
            self.lovebutton.set_sensitive(True)
        except pylast.WSError:
            lib.guihelpers.ErrorMessage(self.window,
                    _("Please check your username and password for Last.fm"))
            CONFIG['lastuser'] = ""
            CONFIG['lastpass'] = ""
            CONFIG['scrobbling'] = 0
            CONFIG.write()
        self.prefs.destroy()

    def close_preferences(self, widget, data = None):
        """
        Closes the Preferences dialog discarding any changes
        """
        self.prefs.destroy()

    def search_grooveshark(self, widget, data = None):
        """
        Starts the search thread
        """
        if widget.get_text() != "":
            if widget.get_text() in CONFIG['completition'].split("|"):
                pass
            else:
                compl = CONFIG['completition'] + widget.get_text() + "|"
                CONFIG['completition'] = compl
                CONFIG.write()
            search_thread = SearchThread(self, widget.get_text(), "Songs")
            search_thread.start()

    def play_selected(self, widget, data = None, row = None):
        """
        Starts the play thread
        """
        if self.get_playing_iter() == None:
            if self.playlist.iter_n_children(None) > 0:
                self.play_by_index(0)
        else:
            self.stop_play()

    def set_songinfos(self, song = None):
        """
        Updates some labels and icons when a song is playing
        """
        if song != None:
            if self.last_iter == None:
                pass
            else:
                name = self.playlist.get_value(self.last_iter, 1)
                self.playlist.set(self.last_iter, 1,
                                  self.create_cornered_image(self.recover_icon(
                                                    self.get_current_cover_url(name))))
            playing = self.get_playing_iter()
            if playing == None:
                playing = self.playlist.get_iter_first()
            path = self.playlist.get_path(playing)
            self.play_view.scroll_to_path(path, False, 0, 0)
            self.last_iter = playing
            self.current_song = song
            self.staticon.change_status_playing()
            image = gtk.Image()
            image.set_from_stock(gtk.STOCK_MEDIA_STOP, gtk.ICON_SIZE_MENU)
            self.staticon.change_playbutton(image)

            image1 = self.playlist.get_value(playing, 1)
            play = gtk.gdk.pixbuf_new_from_file('%s/data/play.png' % BASEPATH)
            play.composite(image1, 0, 0, image1.props.width, image1.props.height,
                              0, 0, 1.0, 1.0, gtk.gdk.INTERP_HYPER, 255)
            self.playlist.set(playing, 1, image1)
            self.songinfo.set_markup(
                _("<b>Playing:</b> {artist} - {title}").format(artist = glib.markup_escape_text(self.current_song['ArtistName']),
                                         title = glib.markup_escape_text(self.current_song['SongName']))
            )
            if int(CONFIG['show_notification']) == 1:
                n = pynotify.Notification(_("Now playing"),
                    "%s - %s" % (glib.markup_escape_text(song['ArtistName']),
                                 glib.markup_escape_text(song['SongName'])),
                    "audio-x-generic")
                n.set_timeout(50)
                n.show()
        else:
            self.staticon.change_status_stopped()
            image = gtk.Image()
            name = self.playlist.get_value(self.last_iter, 1)
            self.playlist.set(self.last_iter, 1,
                              self.create_cornered_image(self.recover_icon(
                                                    self.get_current_cover_url(name))))
            image.set_from_stock(gtk.STOCK_MEDIA_PLAY, gtk.ICON_SIZE_MENU)
            self.staticon.change_playbutton(image)
            self.songinfo.set_markup("")

    def get_overwritten_filename(self, filename):
        """
        Gets the file name depending on whether user want to overwrite the file,
        not overwrite it or not save it. Returns the same filename if user want
        to overwrite, the renamed filename or None if the user cancel Download.
        """
        if os.path.exists(filename) == True:
            dialog = gtk.Dialog(_("Overwrite downloaded file?"),
                         self.window,
                         gtk.DIALOG_MODAL | gtk.DIALOG_DESTROY_WITH_PARENT,
                         (_("Cancel download"), gtk.RESPONSE_NONE,
                          _("Download and overwrite"), 0,
                          _("Download renamed"), 1))

            label = gtk.Label(_("The file '%s' already exists, maybe you are downloading a file that you downloaded before.\n\nWhat do you want to do?") % filename)
            label.set_line_wrap(True)
            dialog.vbox.pack_start(label)
            label.show()

            dialog.set_default_response(gtk.RESPONSE_NONE)
            dialog.set_resizable(False)
            dialog.set_size_request(600, 175)
            response = dialog.run()
            dialog.destroy()
        else:
            response = 0

        if response == 1:
            i = 1
            newfilename = ""
            while i == 1 or os.path.exists(newfilename):
                i += 1
                split = os.path.splitext(filename)
                newfilename = split[0] + " (%d)" % i + split[1]

            return newfilename
        elif response == 0:
            return filename
        else:
            return None

    def download_selected(self, widget, data = None):
        """
        Starts the download thread
        """
        select = self.result_view.get_selection().get_selected_rows()
        for path in select[1]:
            song = self.result_song(path[0])

            #filename = "%s - %s.mp3" % (song["ArtistName"].strip("<>:\"/\|?*"),
            #                        song["SongName"].strip("<>:\"/\|?*"))
            pattern = CONFIG['file_pattern'] + ".mp3"
            filename = pattern.format(artist = song['ArtistName'].strip("<>:\"/\|?&*"),
                                      song = song["SongName"].strip("<>:\"/\|?&*"),
                                      album = song["AlbumName"].strip("<>:\"/\|?&*"))
            filename = filename.replace('/', '-')
            filename = os.path.join(CONFIG['down_path'], filename)
            filename = self.get_overwritten_filename(filename)

            if filename != None:
                try:
                    tDownload = DownloadThread(self, song, filename)
                    tDownload.start()
                except:
                    pass

    def update_downloads_count(self):
        """
        Update the label wich indicates the downloads count.
        """
        count = self.downloads.iter_n_children(None)
        text = gettext.ngettext("Downloading %d file", "Downloading %d files", count) % count
        self.downloads_count.set_label(text)

    def right_click_download(self, treeview, event):
        """
        Executed when the user rightclicks on selected items in the download list.
        Can handle single and multiple selection.
        """
        if event.button == 3:
            path = treeview.get_path_at_pos(int(event.x), int(event.y))
            selection = treeview.get_selection()
            rows = selection.get_selected_rows()
            try:
                if path[0] not in rows[1]:
                    # Unselect if the user click the right button outside 
                    # the selected items - empty field.
                    selection.unselect_all()
                    selection.select_path(path[0])
            except TypeError:
                pass

            # still not sure about this. The point is that there are many rows selected
            # when you click the right mouse button, but then on_cancel_download still 
            # has no idea what is selected.
            if selection.count_selected_rows() > 1:
                self.downmenu.popup(None, None, None, event.button, event.time)
            else:
                self.downmenu.popup(None, None, None, event.button, event.time)

    def cancel_download(self, path, option = None):
        """
        Cancel a download by a list path
        """
        thread = self.downloads[path[0]][4]
        thread.stop(option)
        # TODO: Should improve the busy waiting
        while(thread.is_alive()):
            pass

    def cancel_all_downloads(self):
        """
        Cancel all downloads
        """
        iter = self.downloads.get_iter_first()
        while iter != None:
            path = self.downloads.get_path(iter)
            iter = self.downloads.iter_next(iter)
            self.cancel_download(path)

    def stop_all_downloads(self):
        """
        Stop all downloads used, when quitting.
        Sets the option to stop for the thread
        """
        for i in range(0, len(self.downqueue)):
            path = self.downloads.get_path(self.downqueue[i]['iter'])
            self.cancel_download(path, 'stop')

    def on_cancel_download(self, menu, data = None):
        """
        Cancel the download. Have to check how to cancel the thread.
        """
        select = self.downloads_view.get_selection().get_selected_rows()
        for path in select[1]:
            self.cancel_download(path, 'cancel')

    def on_stop_download(self, button, data = None):
        """
        TODO: Not yet implemented, have to see how the download
        list will be saved.
        Stop download, in order to be resumed later
        """
        select = self.downloads_view.get_selection().get_selected_rows()
        for path in select[1]:
            self.cancel_download(path, 'stop')

    def on_resume_download(self, button, data = None):
        """
        TODO: Not yet implemnted, after stopping download we have to
        load the download list in order to resume.
        Resume stopped download
        """
        select = self.downloads_view.get_selection().get_selected_rows()
        for path in select[1]:
            for i in range(0, len(self.downqueue)):
                if path == self.downloads.get_path(self.downqueue[i]['iter']):
                    tDownload = DownloadThread(self,
                                               self.downqueue[i]['song'],
                                               self.downqueue[i]['filename'],
                                               0, i)
                    tDownload.start()

        #tDownload = DownloadThread(self, self.downqueue[], filename)
        #tDownload.start()

    def get_stream_url(self):
        """
        TODO: Not used at the momment, but should be implemented
        with an short url to share with friends.
        """
        pass

    def on_love_song(self, button):
        if self.current_song == None:
            info = lib.guihelpers.InfoDialog(self.window,
                    _("There is no song to be loved.\n A song should be playing"))
            info.show_all()
        else:
            try:
                track = self.lastfm.get_track(self.current_song['ArtistName'],
                                          self.current_song['SongName'])
                track.love()
            except pylast.MalformedResponseError, e:
                pass

    def get_sliced_string(self, str, max):
        sliced = str[0:max - 3]
        if len(sliced) < len(str):
            return sliced + "..."
        else:
            return str

class SingleService(dbus.service.Object):
    """
    Class to ensure that gSharkDown is running as a single service
    and allow command line arguments to be passed to the application
    """

    def __init__(self, app):
        self.app = app
        bus_name = dbus.service.BusName('org.gsharkdown.Single',
                                       bus = dbus.SessionBus())
        dbus.service.Object.__init__(self, bus_name, '/org/gsharkdown/Single')

    @dbus.service.method(dbus_interface = 'org.gsharkdown.Single')
    def get_current(self):
        """
        Method to get the current playing song used in the
        commandline interface.
        """
        if self.app.current_song == None:
            return "gSharkDown is not playing"
        else:
            song = self.app.current_song['ArtistName']
            song = song + " - "
            song = song + self.app.current_song['SongName']
            return song

    @dbus.service.method(dbus_interface = 'org.gsharkdown.Single')
    def get_state(self):
        """
        Method to get the player state used in the commandline interface
        """
        if self.app.playing == None:
            return "Stopped"
        else:
            return "Playing"

    @dbus.service.method(dbus_interface = 'org.gsharkdown.Single')
    def get_info(self):
        """
        Method to get information about the current playing song
        artist, song, album, year. Used in the commandline interface
        """
        if self.app.current_song == None:
            return ""
        else:
            song = self.app.current_song
            info = "Artist: " + glib.markup_escape_text(song['ArtistName']) + "\n"
            info += "Song:" + glib.markup_escpate_text(song['SongName']) + "\n"
            info += "Album:" + glib.markup_escpate_text(song['AlbumName']) + "\n"
            info += "Year:" + song['Year'] + "\n"
            return info

    @dbus.service.method(dbus_interface = 'org.gsharkdown.Single')
    def get_version(self):
        """
        Method to return the current version of gSharkDown.
        """
        f = open("%s/VERSION" % BASEPATH, 'r')
        version = f.read()
        f.close()
        return version

    @dbus.service.method(dbus_interface = 'org.gsharkdown.Single')
    def get_help(self):
        """
        Method to read the HELP file and display the help message
        when using the commandline interface.
        """
        return open("%s/HELP" % BASEPATH, 'r').read()

if __name__ == "__main__":
    owner = dbus.bus.REQUEST_NAME_REPLY_PRIMARY_OWNER
    if dbus.SessionBus().request_name("org.gsharkdown.Single") != owner:
        """
        This is just a simple salution so gsharkdown can have a 
        commandline interface, so others programs can interract with
        the application and share some informations.
        """
        try:
            if sys.argv[1] in ["--current", "-c", "current"]:
                method = dbus.SessionBus().get_object("org.gsharkdown.Single",
                            "/org/gsharkdown/Single").get_dbus_method("get_current")
            elif sys.argv[1] in ["--state", "-s", "state"]:
                method = dbus.SessionBus().get_object("org.gsharkdown.Single",
                            "/org/gsharkdown/Single").get_dbus_method("get_state")
            elif sys.argv[1] in ["--info", "-i", "info"]:
                method = dbus.SessionBus().get_object("org.gsharkdown.Single",
                            "/org/gsharkdown/Single").get_dbus_method("get_info")
            elif sys.argv[1] in ["--version", "-v", "version"]:
                method = dbus.SessionBus().get_object("org.gsharkdown.Single",
                            "/org/gsharkdown/Single").get_dbus_method("get_version")
            elif sys.argv[1] in ["--help", "-h", "help"]:
                method = dbus.SessionBus().get_object("org.gsharkdown.Single",
                            "/org/gsharkdown/Single").get_dbus_method("get_help")
            print method()
        except:
            print open("%s/HELP" % BASEPATH, 'r').read()
    else:
        app = SharkDown()
        service = SingleService(app)
        init_thread = InitThread(app)
        init_thread.run()
        gtk.main()
