import os
import sys
import gtk
import dbus
import dbus.service
import dbus.mainloop.glib
import time
import threading
import groove
import lyrdblib
import guihelpers
import urllib2
import gobject

import locale
import gettext
from gettext import gettext as _

try:
    import pygst
    pygst.require('0.10')
    import gst
except:
    print "You don't have python-gstreamer installed!"
    sys.exit(0)

try:
    import pycurl
except:
    print "You don't have python-pycurl installed!"
    sys.exit(0)

basepath = os.path.abspath(os.path.dirname(sys.argv[0]).strip("/lib"))

APP = 'gsharkdown'
if os.path.exists("locale"):
    DIR = "%s/locale" % basepath
else:
    DIR = os.path.join(sys.prefix, 'share', 'locale')

try:
    locale.setlocale(locale.LC_ALL, '')
except locale.Error:
    locale.setlocale(locale.LC_ALL, 'en_US.utf8')
locale.bindtextdomain(APP, DIR)
gettext.bindtextdomain(APP, DIR)
gettext.textdomain(APP)
gettext.install(APP, localedir = DIR, unicode = True)


class InitThread(threading.Thread):
    def __init__(self, _frame):
        threading.Thread.__init__(self)
        self._stop = threading.Event()
        self.frame = _frame

    def run(self):
        p = 1
        while p:
            try:
                groove.init()
                groove.getToken()
                p = 0
            except Exception, e:
                if e.args[0] == 11004:
                    time.sleep(1)
                else:
                    error = guihelpers.ErrorMessage(None,
                                    "grooveshark service has probably changed!\n" +
                                    "gSharkDown will not function propperly, so please" +
                                    "be patient till we find a sallution")
                    error.show_all()
                    p = 0
                    self.stop()
    def stop(self):
        self._stop.set()

    def stopped(self):
        return self._stop.isSet()


class DownloadThread(threading.Thread):

    def __init__(self, _window, _song, _filename, _speed = 0, _continue = None):
        """
        _window: The main window object
        _song: The song to download
        _filename: The full path file name to download the song
        """
        threading.Thread.__init__(self)

        self.win = _window

        self.speed = _speed
        self.option = None
        self.cont = _continue

        self.filename = _filename

        try:
            if self.cont == None:
                self.file = open(self.filename, "w")
            else:
                self.file = open(self.filename, "ab")
        except IOError:
            error = guihelpers.ErrorMessage(None,
                _("Failed to create '%s' for writing.") % self.filename)
            error.show_all()
            raise

        self.song = _song
        self.songid = _song["SongID"]
        self._stop = threading.Event()

        if self.cont == None:
            self.iter = self.win.downloads.append([unicode(os.path.basename(self.filename), errors = 'replace'),
                                               self.songid, 0, "", self])
            self.win.downqueue.append({'iter': self.iter, 'song': self.song, 'filename': self.filename})
        else:
            self.iter = self.win.downqueue[self.cont]['iter']

        self.win.update_downloads_count()

    def run(self):
        try:
            key = groove.getStreamKeyFromSongIDEx(self.songid)
        except Exception, e:
            print e
            return

        try:
            url = "http://" + key["result"]["%s" % self.songid]["ip"] + "/stream.php"
            url = str(url)
            c = pycurl.Curl()
            c.setopt(pycurl.URL, url)
            c.setopt(pycurl.NOPROGRESS, 0)
            c.setopt(pycurl.PROGRESSFUNCTION, self.hook)
            c.setopt(pycurl.FAILONERROR, True)
            if self.cont == None:
                c.setopt(pycurl.FILE, self.file)
            else:
                c.setopt(pycurl.RESUME_FROM, os.path.getsize(self.filename))
                c.setopt(pycurl.WRITEDATA, self.file)
            c.setopt(pycurl.POST, True)
            c.setopt(pycurl.NOSIGNAL, True)
            if self.speed != 0:
                c.setopt(pycurl.MAX_RECV_SPEED_LARGE, self.speed)
            c.setopt(pycurl.POSTFIELDS, str("streamKey=" + key["result"]["%s" % self.songid]["streamKey"]))
            c.perform()
            self.file.close()
        except pycurl.error, e:
            if e[0] == 42 and self.option == "stop":
                self.file.close()
            else:
                self.file.close()
                os.remove(self.filename)
        except Exception, e:
            print "Exception ", e
            self.file.close()
            os.remove(self.filename)
            print e.args
        finally:
            #self._remove_download()
            if self.option == "stop":
                pass
            else:
                gobject.idle_add(self._remove_download)

    def stop(self, option = 'cancel'):
        # TODO: This one should be checked how to resolve, at the moment
        # stopping the thread terminates the program
        self.option = option
        self._stop.set()

    def stopped(self):
        return self._stop.isSet()

    def hook(self, downloadTotal, downloadCurrent, uploadTotal, uploadCurrent):
        if self.iter != None and downloadTotal > 0:
            progress = (downloadCurrent / downloadTotal) * 100

            if progress < 99:
                mb = 1024 ** 2
                self.set_value(2, progress)
                self.set_value(3, "%.02f MB" % (downloadTotal / mb))

        if self._stop.isSet():
            return True
        else:
            return False

    def _remove_download(self):
        self.win.downloads.remove(self.iter)
        for i in range(len(self.win.downqueue)):
            if self.iter == self.win.downqueue[i]['iter']:
                self.win.downqueue.pop(i)
        self.iter = None
        self.win.update_downloads_count()

    def set_value(self, column, value):
        try:
            self.win.downloads.set_value(self.iter, column, value)
        except:
            pass


class PlayThread(threading.Thread):

    def __init__(self, _window, _song, _conf, _lastfm):
        """
        Start the player thread with the following parameters:
        @param _window : The main window, to work with in the 
                         main class that would be self
        @param _song : The song information that we get from the search
        @param _conf : The information if the track is to be repeated
        @param _lastfm : The LastFM instance
        """
        threading.Thread.__init__(self)
        self.win = _window
        self.songid = _song["SongID"]
        self.song = _song
        self.repeat = _conf
        self.lastfm = _lastfm

    def run(self):
        try:
            key = groove.getStreamKeyFromSongIDEx(self.songid)
        except Exception, e:
            print e
        playurls = "http://%s/stream.php?streamKey=%s"
        play_url = playurls % (key["result"]["%s" % self.songid]["ip"],
                               key["result"]["%s" % self.songid]["streamKey"])
        self.win.player.set_property('uri', play_url)
        self.win.player.set_state(gst.STATE_PLAYING)
        self.win.play_button.set_stock_id(gtk.STOCK_MEDIA_STOP)
        if self.lastfm != "":
            if self.win.scrobbled == 0:
                self.lastfm.scrobble(self.song['ArtistName'],
                                    self.song['SongName'],
                                    int(time.time()))
                self.lastfm.update_now_playing(self.song['ArtistName'],
                                              self.song['SongName'])
                self.win.scrobbled = 1
            else:
                pass
        bus = self.win.player.get_bus()
        bus.enable_sync_message_emission()
        bus.add_signal_watch()
# I will leave this commented out just till it's confirmed that
# it's not needed anymore.
#        bus.connect("message", self.on_message_cb)
        bus.connect('message::eos', self.on_stopped)

#    def on_message_cb(self, bus, message):
#        if message.type == gst.MESSAGE_STATE_CHANGED:
#            old, new, pending = message.parse_state_changed()
#            self.update_buttons(new)
#        return gst.BUS_PASS

    def on_stopped(self, config, widget = None, data = None):
        """
        Called when the player reaches the end of the stream
        sets state gst.STATE_NULL and calls the play_next 
        method.
        """
        self.win.player.set_state(gst.STATE_NULL)
        self.win.play_button.set_stock_id(gtk.STOCK_MEDIA_STOP)
        self.win.play_next(self.win.ff_button)

#    def update_buttons(self, state):
#        if state == gst.STATE_NULL:
#            self.win.play_button.set_stock_id(gtk.STOCK_MEDIA_PLAY)
#        else:
#            self.win.play_button.set_stock_id(gtk.STOCK_MEDIA_STOP)


class KeyListenerThread(threading.Thread):

    def __init__(self, _window):
        """
        A thread to listen for multimedia key press
        Accepts as parameter:
        @param _window : The main window, to interact with
        """
        threading.Thread.__init__(self)
        self.win = _window

    def run(self):
        """
        Run as a dbus deamon
        """
        try:
            dbus.mainloop.glib.DBusGMainLoop(set_as_default = True)
            bus = dbus.Bus(dbus.Bus.TYPE_SESSION)
            bus_object = bus.get_object('org.gnome.SettingsDaemon',
                                '/org/gnome/SettingsDaemon/MediaKeys')

            dbus_interface = 'org.gnome.SettingsDaemon.MediaKeys'
            bus_object.GrabMediaPlayerKeys("MyMultimediaThingy", 0,
                                  dbus_interface = dbus_interface)

            bus_object.connect_to_signal('MediaPlayerKeyPressed',
                                self.on_media_key)
        except:
            pass

    def on_media_key(self, comes_from, what):
        """
        If a key was press see where it comes from and what key it is
        """
        if what in ['Stop', 'Play', 'Next', 'Previous']:
            if what == 'Stop':
                self.win.play_selected(self.win.play_button)
            elif what == 'Play':
                self.win.play_selected(self.win.play_button)
            elif what == 'Next':
                self.win.play_next(self.win.ff_button)
            elif what == 'Previous':
                self.win.play_previous(self.win.prev_button)

class SearchThread(threading.Thread):

    def __init__(self, _window, _term, _type):
        """
        A search thread class accepts as parameters:
        @param _window : The main window to interact with
        @param _term : The search term that was put in
        @param _type : Type of the search to be done
        """
        threading.Thread.__init__(self)
        self.win = _window
        self.term = _term
        self.type = _type

    def run(self):
        self.win.entry.set_sensitive(False)
        try:
            results = groove.getSearchResultsEx(self.term, self.type)
            self.win.entry.set_sensitive(True)
            self.win.result.clear()
            if results:
                for item in results:
                    self.win.result.append([item['SongName'],
                                    item['ArtistName'],
                                    item['AlbumName'],
                                    item['SongID'],
                                    item])
        except Exception, e:
            print e.args


class UpdateThread(threading.Thread):

    def __init__(self, _basepath, _updatemsg, _noupdate, _secondary):
        """
        Thread for comparing the local version with the remote. Accepts 
        the following parameters:
        @param _basepath : path to the base directory of gSharkDown
        @param _updatemsg : Message if new version is up
        @param _noupdate : Message if no new version is up
        @param _secondary : Secondary text for explanation
        """
        threading.Thread.__init__(self)
        self.path = _basepath
        self.updatemsg = _updatemsg
        self.noupdate = _noupdate
        self.secondary = _secondary
        self.checkurl = 'http://bitbucket.org/vkolev/gsharkdown/raw/tip/VERSION'

    def run(self):
        local = open("%s/VERSION" % self.path, 'r')
        localversion = local.read().replace('\n', '')
        remote = urllib2.urlopen(self.checkurl)
        remoteversion = remote.read().replace('\n', '')
        if remoteversion > localversion:
            dialog = guihelpers.UpdateDialog(_('New version'),
                                             self.updatemsg % remoteversion,
                                             self.secondary)
            dialog.show_all()
        else:
            dialog = guihelpers.UpdateDialog(_('Latest version'),
                                             self.noupdate,
                                             self.secondary)
            dialog.show_all()


class LyricsThread(threading.Thread):

    def __init__(self, _parent, _uifile, _artist, _song):
        """
        Thread for searching lyrics for a specified song with given
        artist and songname.
        """
        threading.Thread.__init__(self)
        self.parent = _parent
        self.ui = _uifile
        self.artist = _artist
        self.song = _song

    def run(self):
        self.parent.lyrics_button.set_sensitive(False)
        self.parent.lyrics_button.set_label(_("Loading lyrics..."))
        lyrics = lyrdblib.search(self.artist, self.song)
        self.parent.lyrics_button.set_sensitive(True)
        self.parent.lyrics_button.set_label(_("Lyrics"))
        if "ERROR:" in lyrics:
            error = guihelpers.ErrorMessage(None,
                lyrics)
            error.show_all()
        else:
            lyrdiag = guihelpers.LyricsDialog(self.ui,
                                              self.artist,
                                              self.song,
                                              lyrics)
            lyrdiag.show_all()

class SongCoverThread(threading.Thread):
    def __init__(self, _parent, _iter):
        threading.Thread.__init__(self)
        self.parent = _parent
        self.iter = _iter

    def run(self):
        cover_url = self.parent.playlist_song(self.iter)['CoverArtFilename']
        pixbuf = None

        try:
            url = "http://beta.grooveshark.com/static/amazonart/s%s"
            response = urllib2.urlopen(url % cover_url)
            loader = gtk.gdk.PixbufLoader()
            loader.write(response.read())
            loader.close()
            image = loader.get_pixbuf()
            pixbuf = self.parent.create_cornered_image(image)
        except:
            pixbuf = self.parent.create_default_track_icon()

        self.parent.playlist[self.iter][1] = pixbuf

